==> node.io-0.2.1-7.tgz: package/lib/node.io/utils.js <==
/*!
 * node.io
 * Copyright(c) 2010 Chris O'Hara <cohara87@gmail.com>
 * MIT Licensed
 */

var cwd = process.cwd(),
    fs = require('fs'),
    daemon = require('daemon'),
    exec = require('child_process').exec;
 
/**
 * Outputs a styled message to the console.
 *
 * @param {String} msg
 * @param {String} type (optional)
 * @api public
 */
var status = function (msg, type) {
    switch (type) {
    case 'info': 
        msg = '\x1B[33mINFO\x1B[0m: ' + msg; 
        break;
        
    case 'debug':
        msg = '\x1B[36mDEBUG\x1B[0m: ' + msg; 
        break;
        
    case 'error': 
    case 'fatal': 
        msg = '\x1B[31mERROR\x1B[0m: ' + msg;
        break;
        
    case 'ok': 
        msg = '\x1B[32mOK\x1B[0m: ' + msg; 
        break; 
        
    case 'bold': 
        msg = '\x1B[33mINFO\x1B[0m: \x1B[1m' + msg + '\x1B[0m'; 
        break; 
    }
    console.log(msg);
    if (type === 'fatal') {
        process.exit(1);
    }
};
exports.status = {};
['info', 'error', 'fatal', 'ok', 'debug', 'bold'].forEach(function (type) {
    exports.status[type] = function (msg) {
        status(msg, type);
    };
});

/**
 * Removes the specified file on exit.
 *
 * @param {String} file
 * @api public
 */
exports.removeOnExit = function (file) {
    var removeFile = function () {
        try {
            fs.unlinkSync(file);
        } catch (e) {}
    };
    ['SIGINT', 'SIGTERM', 'SIGKILL', 'SIGQUIT', 'SIGHUP', 'exit'].forEach(function (signal) {
        process.on(signal, removeFile);
    });
};

/**
 * Compiles a CoffeeScript file (.coffee) using coffee(1).
 *
 * @param {String} coffee_file
 * @param {String} compiled_file
 * @param {Function} callback
 * @api public
 */
exports.compileCoffee = function (coffee_file, compiled_file, callback) {
    exec('coffee -p -c "' + coffee_file + '"', {cwd: cwd}, function (err, stdout, stderr) {
        if (err || stderr) {
            callback(err || stderr);
        } else {
            exports.removeOnExit(compiled_file);
            fs.writeFile(compiled_file, stdout, callback);
        }
    });
};

/**
 * Calculates the CRC32 checksum of a string.
 *
 * @param {String} str
 * @return {String} checksum
 * @api public
 */
exports.crc32 = function (str) {
    var crc32tab = [
        0x00000000, 0x77073096, 0xee0e612c, 0x990951ba,
        0x076dc419, 0x706af48f, 0xe963a535, 0x9e6495a3,
        0x0edb8832, 0x79dcb8a4, 0xe0d5e91e, 0x97d2d988,
        0x09b64c2b, 0x7eb17cbd, 0xe7b82d07, 0x90bf1d91,
        0x1db71064, 0x6ab020f2, 0xf3b97148, 0x84be41de,
        0x1adad47d, 0x6ddde4eb, 0xf4d4b551, 0x83d385c7,
        0x136c9856, 0x646ba8c0, 0xfd62f97a, 0x8a65c9ec,
        0x14015c4f, 0x63066cd9, 0xfa0f3d63, 0x8d080df5,
        0x3b6e20c8, 0x4c69105e, 0xd56041e4, 0xa2677172,
        0x3c03e4d1, 0x4b04d447, 0xd20d85fd, 0xa50ab56b,
        0x35b5a8fa, 0x42b2986c, 0xdbbbc9d6, 0xacbcf940,
        0x32d86ce3, 0x45df5c75, 0xdcd60dcf, 0xabd13d59,
        0x26d930ac, 0x51de003a, 0xc8d75180, 0xbfd06116,
        0x21b4f4b5, 0x56b3c423, 0xcfba9599, 0xb8bda50f,
        0x2802b89e, 0x5f058808, 0xc60cd9b2, 0xb10be924,
        0x2f6f7c87, 0x58684c11, 0xc1611dab, 0xb6662d3d,
        0x76dc4190, 0x01db7106, 0x98d220bc, 0xefd5102a,
        0x71b18589, 0x06b6b51f, 0x9fbfe4a5, 0xe8b8d433,
        0x7807c9a2, 0x0f00f934, 0x9609a88e, 0xe10e9818,
        0x7f6a0dbb, 0x086d3d2d, 0x91646c97, 0xe6635c01,
        0x6b6b51f4, 0x1c6c6162, 0x856530d8, 0xf262004e,
        0x6c0695ed, 0x1b01a57b, 0x8208f4c1, 0xf50fc457,
        0x65b0d9c6, 0x12b7e950, 0x8bbeb8ea, 0xfcb9887c,
        0x62dd1ddf, 0x15da2d49, 0x8cd37cf3, 0xfbd44c65,
        0x4db26158, 0x3ab551ce, 0xa3bc0074, 0xd4bb30e2,
        0x4adfa541, 0x3dd895d7, 0xa4d1c46d, 0xd3d6f4fb,
        0x4369e96a, 0x346ed9fc, 0xad678846, 0xda60b8d0,
        0x44042d73, 0x33031de5, 0xaa0a4c5f, 0xdd0d7cc9,
        0x5005713c, 0x270241aa, 0xbe0b1010, 0xc90c2086,
        0x5768b525, 0x206f85b3, 0xb966d409, 0xce61e49f,
        0x5edef90e, 0x29d9c998, 0xb0d09822, 0xc7d7a8b4,
        0x59b33d17, 0x2eb40d81, 0xb7bd5c3b, 0xc0ba6cad,
        0xedb88320, 0x9abfb3b6, 0x03b6e20c, 0x74b1d29a,
        0xead54739, 0x9dd277af, 0x04db2615, 0x73dc1683,
        0xe3630b12, 0x94643b84, 0x0d6d6a3e, 0x7a6a5aa8,
        0xe40ecf0b, 0x9309ff9d, 0x0a00ae27, 0x7d079eb1,
        0xf00f9344, 0x8708a3d2, 0x1e01f268, 0x6906c2fe,
        0xf762575d, 0x806567cb, 0x196c3671, 0x6e6b06e7,
        0xfed41b76, 0x89d32be0, 0x10da7a5a, 0x67dd4acc,
        0xf9b9df6f, 0x8ebeeff9, 0x17b7be43, 0x60b08ed5,
        0xd6d6a3e8, 0xa1d1937e, 0x38d8c2c4, 0x4fdff252,
        0xd1bb67f1, 0xa6bc5767, 0x3fb506dd, 0x48b2364b,
        0xd80d2bda, 0xaf0a1b4c, 0x36034af6, 0x41047a60,
        0xdf60efc3, 0xa867df55, 0x316e8eef, 0x4669be79,
        0xcb61b38c, 0xbc66831a, 0x256fd2a0, 0x5268e236,
        0xcc0c7795, 0xbb0b4703, 0x220216b9, 0x5505262f,
        0xc5ba3bbe, 0xb2bd0b28, 0x2bb45a92, 0x5cb36a04,
        0xc2d7ffa7, 0xb5d0cf31, 0x2cd99e8b, 0x5bdeae1d,
        0x9b64c2b0, 0xec63f226, 0x756aa39c, 0x026d930a,
        0x9c0906a9, 0xeb0e363f, 0x72076785, 0x05005713,
        0x95bf4a82, 0xe2b87a14, 0x7bb12bae, 0x0cb61b38,
        0x92d28e9b, 0xe5d5be0d, 0x7cdcefb7, 0x0bdbdf21,
        0x86d3d2d4, 0xf1d4e242, 0x68ddb3f8, 0x1fda836e,
        0x81be16cd, 0xf6b9265b, 0x6fb077e1, 0x18b74777,
        0x88085ae6, 0xff0f6a70, 0x66063bca, 0x11010b5c,
        0x8f659eff, 0xf862ae69, 0x616bffd3, 0x166ccf45,
        0xa00ae278, 0xd70dd2ee, 0x4e048354, 0x3903b3c2,
        0xa7672661, 0xd06016f7, 0x4969474d, 0x3e6e77db,
        0xaed16a4a, 0xd9d65adc, 0x40df0b66, 0x37d83bf0,
        0xa9bcae53, 0xdebb9ec5, 0x47b2cf7f, 0x30b5ffe9,
        0xbdbdf21c, 0xcabac28a, 0x53b39330, 0x24b4a3a6,
        0xbad03605, 0xcdd70693, 0x54de5729, 0x23d967bf,
        0xb3667a2e, 0xc4614ab8, 0x5d681b02, 0x2a6f2b94,
        0xb40bbe37, 0xc30c8ea1, 0x5a05df1b, 0x2d02ef8d
    ];
    
    var crc = ~0;
    for (var i = 0; i < str.length; i++) {
        crc = (crc >>> 8) ^ crc32tab[(crc ^ str.charCodeAt(i)) & 0xff];
    }

    crc = (~crc >>> 16) & 0x7fff;
    return crc === 0 ? 1 : crc;
};

/**
 * Takes an arbitrary number of arguments (objects) and copies
 * ALL properties from right to left.
 *
 * @param {Object(s)} ...
 * @return {Object} 
 * @api public
 */
 var x = 0;
exports.put = function (o /*, args */) {
    //Create o={} if not an object
    if (typeof o !== 'object') o = {};
    
    var obj = Array.prototype.slice.call(arguments,1),
        i = 0, l = obj.length;
    
    //Iterate over each subsequent object and copy properties into o    
    for (; i < l; i++) {
        for (var j in obj[i]) {
            if (obj[i].hasOwnProperty(j)) {
                o[j] = obj[i][j];
            }
        }
    }
        
    return o;
}

/**
 * Takes an arbitrary number of arguments (objects) and copies
 * properties from right to left only if they don't already exist.
 *
 * @param {Object(s)} ...
 * @return {Object} 
 * @api public
 */
exports.put_default = function (o /*, args */) {
    //Create o={} if not an object
    if (typeof o !== 'object') o = {};
        
    var obj = Array.prototype.slice.call(arguments,1),
        i = 0, l = obj.length;
    
    //Iterate over each subsequent object and copy properties into o
    //if they don't already exist
    for (; i < l; i++) {
        for (var j in obj[i]) {
            if (typeof o[j] === 'undefined' && obj[i].hasOwnProperty(j)) {
                o[j] = obj[i][j];
            }
        }
    }
        
    return o;
}

/**
 * Custom stringify function.
 *
 * @param {Object} data
 * @param {String} newline (optional)
 * @param {Boolean} stringify_array
 * @return {String} 
 * @api public
 */
exports.dataToString = function (data, newline, stringify_array) {
    var str = '';
    newline = newline || '\n';
    if (!stringify_array && data instanceof Array) {
        for (var i = 0, l = data.length; i < l; i++) {
            str += exports.dataToString(data[i], newline, true);
        }
    } else if (typeof data === 'object') {
        str = JSON.stringify(data) + newline;
    } else {
        str = data + newline;
    }
    return str;
};

/**
 * A method for creating and controlling a node.io daemon.
 *
 * `arg` can be:
 *      start = daemonizes the process
 *      stop  = stops the daemon if it is running
 *      restart = alias for stop -> start
 *      pid = outputs the daemon's PID if it is running
 *      log = outputs the daemon's log file (stdout & stderr)
 *
 * @param {String} arg
 * @param {Function} callback
 * @api public
 */
exports.daemonize = function (arg, callback) {
    var lock_file = '/tmp/nodeio.pid',
        log_file = '/tmp/nodeio.log';
            
    var start = function () {
        daemon.run(log_file, lock_file, function (err) {
            if (err) return status('Error starting daemon: ' + err, 'error');
            callback();
        });
    };
    
    var stop = function () {
        try {
            fs.readFileSync(lock_file);
        } catch (e) {
            return status('Daemon is not running', 'error');
        };
        daemon.stop(lock_file, function (err, pid) {
            if (err && err.errno === 3) {
                return status('Daemon is not running', 'error');
            } else if (err) {
                return status('Error stopping daemon: ' + err.errno, 'error');
            }
            status('Successfully stopped daemon with pid: ' + pid, 'ok');
        });
    };
    
    switch(arg) {
	case 'stop':
		stop();
		break;

	case 'restart':
        daemon.stop(lock_file, function () {
            start();
        });
		break;
    
    case 'log':
        try {
            console.log(fs.readFileSync(log_file, 'utf8'));
        } catch (e) {
            return status('No daemon log file', 'error');
        };
        break;
    
    case 'pid':
        try {
            var pid = fs.readFileSync(lock_file, 'utf8');
            fs.statSync('/proc/' + pid);
            status(pid, 'info');
        } catch (e) {
            return status('Daemon is not running', 'error');
        };
        break;
    
	default:
        start();
        break;
    }
}

/**
 * Recursively find all files in a directory (async).
 *
 * `callback` receives an array of files.
 * 
 * @param {String} path
 * @param {Function} callback
 * @api public
 */
exports.getFiles = function (root_path, callback) {
    var files = [];
    fs.readdir(root_path, function(err, dir_files) {
        if (err) throw err;
        var i = dir_files.length;
        var check_complete = function () {
            if (i === 0) {
                callback(files);
            }
        };
        check_complete();
        dir_files.forEach(function (path) {
            fs.stat(root_path + '/' + path, function (err, stat) {
                if (err) throw err;
                if (stat.isDirectory()) {
                    exports.getFiles(root_path + '/' + path, function (subdir_files) {
                        subdir_files.forEach(function (file) {
                            files.push(path + '/' + file);
                        });
                        i--;
                        check_complete();
                    });
                } else {
                    files.push(path);
                    i--;
                    check_complete();
                }
            });
        });
    });
};

==> node.io-0.2.1-10.tgz: package/lib/node.io/utils.js <==
/*!
 * node.io
 * Copyright(c) 2010 Chris O'Hara <cohara87@gmail.com>
 * MIT Licensed
 */

var cwd = process.cwd(),
    fs = require('fs'),
    daemon = require('daemon'),
    exec = require('child_process').exec;
 
/**
 * Outputs a styled message to the console.
 *
 * @param {String} msg
 * @param {String} type (optional)
 * @api public
 */
var status = function (msg, type) {
    switch (type) {
    case 'info': 
        msg = '\x1B[33mINFO\x1B[0m: ' + msg; 
        break;
        
    case 'debug':
        msg = '\x1B[36mDEBUG\x1B[0m: ' + msg; 
        break;
        
    case 'error': 
    case 'fatal': 
        msg = '\x1B[31mERROR\x1B[0m: ' + msg;
        break;
        
    case 'ok': 
        msg = '\x1B[32mOK\x1B[0m: ' + msg; 
        break; 
        
    case 'bold': 
        msg = '\x1B[33mINFO\x1B[0m: \x1B[1m' + msg + '\x1B[0m'; 
        break; 
    }
    console.log(msg);
    if (type === 'fatal') {
        process.exit(1);
    }
};
exports.status = {};
['info', 'error', 'fatal', 'ok', 'debug', 'bold'].forEach(function (type) {
    exports.status[type] = function (msg) {
        status(msg, type);
    };
});

/**
 * Removes the specified file on exit.
 *
 * @param {String} file
 * @api public
 */
exports.removeOnExit = function (file) {
    var removeFile = function () {
        try {
            fs.unlinkSync(file);
        } catch (e) {}
    };
    ['SIGINT', 'SIGTERM', 'SIGKILL', 'SIGQUIT', 'SIGHUP', 'exit'].forEach(function (signal) {
        process.on(signal, removeFile);
    });
};

/**
 * Compiles a CoffeeScript file (.coffee) using coffee(1).
 *
 * @param {String} coffee_file
 * @param {String} compiled_file
 * @param {Function} callback
 * @api public
 */
exports.compileCoffee = function (coffee_file, compiled_file, callback) {
    exec('coffee -p -c "' + coffee_file + '"', {cwd: cwd}, function (err, stdout, stderr) {
        if (err || stderr) {
            callback(err || stderr);
        } else {
            exports.removeOnExit(compiled_file);
            fs.writeFile(compiled_file, stdout, callback);
        }
    });
};

/**
 * Calculates the CRC32 checksum of a string.
 *
 * @param {String} str
 * @return {String} checksum
 * @api public
 */
var crc32tab = [
    0x00000000, 0x77073096, 0xee0e612c, 0x990951ba,
    0x076dc419, 0x706af48f, 0xe963a535, 0x9e6495a3,
    0x0edb8832, 0x79dcb8a4, 0xe0d5e91e, 0x97d2d988,
    0x09b64c2b, 0x7eb17cbd, 0xe7b82d07, 0x90bf1d91,
    0x1db71064, 0x6ab020f2, 0xf3b97148, 0x84be41de,
    0x1adad47d, 0x6ddde4eb, 0xf4d4b551, 0x83d385c7,
    0x136c9856, 0x646ba8c0, 0xfd62f97a, 0x8a65c9ec,
    0x14015c4f, 0x63066cd9, 0xfa0f3d63, 0x8d080df5,
    0x3b6e20c8, 0x4c69105e, 0xd56041e4, 0xa2677172,
    0x3c03e4d1, 0x4b04d447, 0xd20d85fd, 0xa50ab56b,
    0x35b5a8fa, 0x42b2986c, 0xdbbbc9d6, 0xacbcf940,
    0x32d86ce3, 0x45df5c75, 0xdcd60dcf, 0xabd13d59,
    0x26d930ac, 0x51de003a, 0xc8d75180, 0xbfd06116,
    0x21b4f4b5, 0x56b3c423, 0xcfba9599, 0xb8bda50f,
    0x2802b89e, 0x5f058808, 0xc60cd9b2, 0xb10be924,
    0x2f6f7c87, 0x58684c11, 0xc1611dab, 0xb6662d3d,
    0x76dc4190, 0x01db7106, 0x98d220bc, 0xefd5102a,
    0x71b18589, 0x06b6b51f, 0x9fbfe4a5, 0xe8b8d433,
    0x7807c9a2, 0x0f00f934, 0x9609a88e, 0xe10e9818,
    0x7f6a0dbb, 0x086d3d2d, 0x91646c97, 0xe6635c01,
    0x6b6b51f4, 0x1c6c6162, 0x856530d8, 0xf262004e,
    0x6c0695ed, 0x1b01a57b, 0x8208f4c1, 0xf50fc457,
    0x65b0d9c6, 0x12b7e950, 0x8bbeb8ea, 0xfcb9887c,
    0x62dd1ddf, 0x15da2d49, 0x8cd37cf3, 0xfbd44c65,
    0x4db26158, 0x3ab551ce, 0xa3bc0074, 0xd4bb30e2,
    0x4adfa541, 0x3dd895d7, 0xa4d1c46d, 0xd3d6f4fb,
    0x4369e96a, 0x346ed9fc, 0xad678846, 0xda60b8d0,
    0x44042d73, 0x33031de5, 0xaa0a4c5f, 0xdd0d7cc9,
    0x5005713c, 0x270241aa, 0xbe0b1010, 0xc90c2086,
    0x5768b525, 0x206f85b3, 0xb966d409, 0xce61e49f,
    0x5edef90e, 0x29d9c998, 0xb0d09822, 0xc7d7a8b4,
    0x59b33d17, 0x2eb40d81, 0xb7bd5c3b, 0xc0ba6cad,
    0xedb88320, 0x9abfb3b6, 0x03b6e20c, 0x74b1d29a,
    0xead54739, 0x9dd277af, 0x04db2615, 0x73dc1683,
    0xe3630b12, 0x94643b84, 0x0d6d6a3e, 0x7a6a5aa8,
    0xe40ecf0b, 0x9309ff9d, 0x0a00ae27, 0x7d079eb1,
    0xf00f9344, 0x8708a3d2, 0x1e01f268, 0x6906c2fe,
    0xf762575d, 0x806567cb, 0x196c3671, 0x6e6b06e7,
    0xfed41b76, 0x89d32be0, 0x10da7a5a, 0x67dd4acc,
    0xf9b9df6f, 0x8ebeeff9, 0x17b7be43, 0x60b08ed5,
    0xd6d6a3e8, 0xa1d1937e, 0x38d8c2c4, 0x4fdff252,
    0xd1bb67f1, 0xa6bc5767, 0x3fb506dd, 0x48b2364b,
    0xd80d2bda, 0xaf0a1b4c, 0x36034af6, 0x41047a60,
    0xdf60efc3, 0xa867df55, 0x316e8eef, 0x4669be79,
    0xcb61b38c, 0xbc66831a, 0x256fd2a0, 0x5268e236,
    0xcc0c7795, 0xbb0b4703, 0x220216b9, 0x5505262f,
    0xc5ba3bbe, 0xb2bd0b28, 0x2bb45a92, 0x5cb36a04,
    0xc2d7ffa7, 0xb5d0cf31, 0x2cd99e8b, 0x5bdeae1d,
    0x9b64c2b0, 0xec63f226, 0x756aa39c, 0x026d930a,
    0x9c0906a9, 0xeb0e363f, 0x72076785, 0x05005713,
    0x95bf4a82, 0xe2b87a14, 0x7bb12bae, 0x0cb61b38,
    0x92d28e9b, 0xe5d5be0d, 0x7cdcefb7, 0x0bdbdf21,
    0x86d3d2d4, 0xf1d4e242, 0x68ddb3f8, 0x1fda836e,
    0x81be16cd, 0xf6b9265b, 0x6fb077e1, 0x18b74777,
    0x88085ae6, 0xff0f6a70, 0x66063bca, 0x11010b5c,
    0x8f659eff, 0xf862ae69, 0x616bffd3, 0x166ccf45,
    0xa00ae278, 0xd70dd2ee, 0x4e048354, 0x3903b3c2,
    0xa7672661, 0xd06016f7, 0x4969474d, 0x3e6e77db,
    0xaed16a4a, 0xd9d65adc, 0x40df0b66, 0x37d83bf0,
    0xa9bcae53, 0xdebb9ec5, 0x47b2cf7f, 0x30b5ffe9,
    0xbdbdf21c, 0xcabac28a, 0x53b39330, 0x24b4a3a6,
    0xbad03605, 0xcdd70693, 0x54de5729, 0x23d967bf,
    0xb3667a2e, 0xc4614ab8, 0x5d681b02, 0x2a6f2b94,
    0xb40bbe37, 0xc30c8ea1, 0x5a05df1b, 0x2d02ef8d
];
exports.crc32 = function (str, hex) {    
    var crc = ~0, i;
    for (i = 0; i < str.length; i++) {
        crc = (crc >>> 8) ^ crc32tab[(crc ^ str.charCodeAt(i)) & 0xff];
    }
    crc = crc ^ (-1);
    return hex ? crc.toString(16) : crc;
};

/**
 * Takes an arbitrary number of arguments (objects) and copies
 * ALL properties from right to left.
 *
 * @param {Object(s)} ...
 * @return {Object} 
 * @api public
 */
 var x = 0;
exports.put = function (o /*, args */) {
    //Create o={} if not an object
    if (typeof o !== 'object') o = {};
    
    var obj = Array.prototype.slice.call(arguments,1),
        i = 0, l = obj.length;
    
    //Iterate over each subsequent object and copy properties into o    
    for (; i < l; i++) {
        for (var j in obj[i]) {
            if (obj[i].hasOwnProperty(j)) {
                o[j] = obj[i][j];
            }
        }
    }
        
    return o;
}

/**
 * Takes an arbitrary number of arguments (objects) and copies
 * properties from right to left only if they don't already exist.
 *
 * @param {Object(s)} ...
 * @return {Object} 
 * @api public
 */
exports.put_default = function (o /*, args */) {
    //Create o={} if not an object
    if (typeof o !== 'object') o = {};
        
    var obj = Array.prototype.slice.call(arguments,1),
        i = 0, l = obj.length;
    
    //Iterate over each subsequent object and copy properties into o
    //if they don't already exist
    for (; i < l; i++) {
        for (var j in obj[i]) {
            if (typeof o[j] === 'undefined' && obj[i].hasOwnProperty(j)) {
                o[j] = obj[i][j];
            }
        }
    }
        
    return o;
}

/**
 * Custom stringify function.
 *
 * @param {Object} data
 * @param {String} newline (optional)
 * @param {Boolean} stringify_array
 * @return {String} 
 * @api public
 */
exports.dataToString = function (data, newline, stringify_array) {
    var str = '';
    newline = newline || '\n';
    if (!stringify_array && data instanceof Array) {
        for (var i = 0, l = data.length; i < l; i++) {
            str += exports.dataToString(data[i], newline, true);
        }
    } else if (typeof data === 'object') {
        str = JSON.stringify(data) + newline;
    } else {
        str = data + newline;
    }
    return str;
};

/**
 * A method for creating and controlling a node.io daemon.
 *
 * `arg` can be:
 *      start = daemonizes the process
 *      stop  = stops the daemon if it is running
 *      restart = alias for stop -> start
 *      pid = outputs the daemon's PID if it is running
 *      log = outputs the daemon's log file (stdout & stderr)
 *
 * @param {String} arg
 * @param {Function} callback
 * @api public
 */
exports.daemonize = function (arg, callback) {
    var lock_file = '/tmp/nodeio.pid',
        log_file = '/tmp/nodeio.log';
            
    var start = function () {
        daemon.run(log_file, lock_file, function (err) {
            if (err) return status('Error starting daemon: ' + err, 'error');
            callback();
        });
    };
    
    var stop = function () {
        try {
            fs.readFileSync(lock_file);
        } catch (e) {
            return status('Daemon is not running', 'error');
        };
        daemon.stop(lock_file, function (err, pid) {
            if (err && err.errno === 3) {
                return status('Daemon is not running', 'error');
            } else if (err) {
                return status('Error stopping daemon: ' + err.errno, 'error');
            }
            status('Successfully stopped daemon with pid: ' + pid, 'ok');
        });
    };
    
    switch(arg) {
	case 'stop':
		stop();
		break;

	case 'restart':
        daemon.stop(lock_file, function () {
            start();
        });
		break;
    
    case 'log':
        try {
            console.log(fs.readFileSync(log_file, 'utf8'));
        } catch (e) {
            return status('No daemon log file', 'error');
        };
        break;
    
    case 'pid':
        try {
            var pid = fs.readFileSync(lock_file, 'utf8');
            fs.statSync('/proc/' + pid);
            status(pid, 'info');
        } catch (e) {
            return status('Daemon is not running', 'error');
        };
        break;
    
	default:
        start();
        break;
    }
}

/**
 * Recursively find all files in a directory (async).
 *
 * `callback` receives an array of files.
 * 
 * @param {String} path
 * @param {Function} callback
 * @api public
 */
exports.getFiles = function (root_path, callback) {
    var files = [];
    fs.readdir(root_path, function(err, dir_files) {
        if (err) throw err;
        var i = dir_files.length;
        var check_complete = function () {
            if (i === 0) {
                callback(files);
            }
        };
        check_complete();
        dir_files.forEach(function (path) {
            fs.stat(root_path + '/' + path, function (err, stat) {
                if (err) throw err;
                if (stat.isDirectory()) {
                    exports.getFiles(root_path + '/' + path, function (subdir_files) {
                        subdir_files.forEach(function (file) {
                            files.push(path + '/' + file);
                        });
                        i--;
                        check_complete();
                    });
                } else {
                    files.push(path);
                    i--;
                    check_complete();
                }
            });
        });
    });
};
